==> py22-numarray-0.3.4.tbz: share/examples/py-numarray/convolve/high_levelmodule.c <==
#include <stdio.h>
#include <math.h>
#include <signal.h>
#include <ctype.h>

#include <Python.h>
#include <libnumarray.h>

static PyObject *_convolveError;

static void 
Convolve1d(long ksizex, Float64 *kernel, 
	   long dsizex, Float64 *data, 
	   Float64 *convolved)
{
	long xc;
	long halfk = ksizex/2;

	for(xc=0; xc<halfk; xc++)
		convolved[xc] = data[xc];

	for(xc=halfk; xc<dsizex-halfk; xc++) {
		long xk;
		double temp = 0;
		for (xk=0; xk<ksizex; xk++)
			temp += kernel[xk]*data[xc-halfk+xk];
		convolved[xc] = temp;
	}
		     
	for(xc=dsizex-halfk; xc<dsizex; xc++)
		convolved[xc] = data[xc];
}

static void 
Convolve2d(long krows, long kcols, Float64 *kernel, 
	   long drows, long dcols, Float64 *data, 
	   Float64 *convolved)
{
	long ki, kj, di, dj;
	long halfkrows = krows/2;
	long halfkcols = kcols/2;

	/* Copy the data in the half kernel "frame" straight through. */
	for(di=0; di<halfkrows; di++) {
		for(dj=0; dj<dcols; dj++)
			convolved[di*dcols+dj] = data[di*dcols+dj];
	}
	for(di=drows-halfkrows; di<drows; di++) {
		for(dj=0; dj<dcols; dj++)
			convolved[di*dcols+dj] = data[di*dcols+dj];
	}
	for(di=halfkrows; di<drows-halfkrows; di++) {
		for(dj=0; dj<halfkcols; dj++)
			convolved[di*dcols+dj] = data[di*dcols+dj];
	}
	for(di=halfkrows; di<drows-halfkrows; di++) {
		for(dj=dcols-halfkcols; dj<dcols; dj++)
			convolved[di*dcols+dj] = data[di*dcols+dj];
	}

	/* Do the convolution on the data within the frame. */
	for(di=halfkrows; di<drows-halfkrows; di++) {
		for(dj=halfkcols; dj<dcols-halfkcols; dj++) {
			Float64 temp = 0;
			for(ki=0; ki<krows; ki++) {
				long pi = di + ki - halfkrows;
				for(kj=0; kj<kcols; kj++) {
					long pj = dj + kj - halfkcols;
					temp += data[pi*dcols+pj] * 
						kernel[ki*kcols+kj];
				}
			}
			convolved[di*dcols+dj] = temp;
		}
	}
}


static PyObject *
Py_Convolve1d(PyObject *obj, PyObject *args)
{
	PyObject   *kernel, *data, *convolved=Py_None, *cshadow;
	NDInfo     kinfo, dinfo, cinfo;

	if (!PyArg_ParseTuple(args, "OO|O", &kernel, &data, &convolved))
		return PyErr_Format(_convolveError, 
				    "Convolve1d: Invalid parameters.");

	/* Align, Byteswap, Contiguous, Typeconvert */
	kernel	= NA_InputArray(kernel, &kinfo, tFloat64, C_ARRAY);
	data	= NA_InputArray(data, &dinfo, tFloat64, C_ARRAY);
	cshadow = NA_OptionalOutputArray(convolved, &cinfo, tFloat64, C_ARRAY, data);

	if (!kernel || !data || !cshadow)
		return 0;

	if ((kinfo.ndim != 1) || (dinfo.ndim != 1))
		return PyErr_Format(_convolveError,
				    "Convolve1d: arrays must have exactly 1 dimension.");

	if (NA_ShapeMisMatch(&dinfo, &cinfo))
		return PyErr_Format(_convolveError,
				    "Convolve1d: data and output arrays must have identical length.");

	Convolve1d(kinfo.nelem, NA_OFFSETDATA(kinfo),
		   dinfo.nelem, NA_OFFSETDATA(dinfo),
		   NA_OFFSETDATA(cinfo));

	Py_DECREF(kernel);
	Py_DECREF(data);

	/* Align, Byteswap, Contiguous, Typeconvert */
	return NA_ReturnOutput(convolved, cshadow);
}


static PyObject *
Py_Convolve2d(PyObject *obj, PyObject *args)
{
	PyObject   *kernel, *data, *convolved=Py_None, *cshadow;
	NDInfo     kinfo, dinfo, cinfo;

	if (!PyArg_ParseTuple(args, "OO|O", &kernel, &data, &convolved))
		return PyErr_Format(_convolveError, 
				    "Convolve2d: Invalid parameters.");

	/* Align, Byteswap, Contiguous, Typeconvert */
	kernel	= NA_InputArray(kernel, &kinfo, tFloat64, C_ARRAY);
	data	= NA_InputArray(data, &dinfo, tFloat64, C_ARRAY);
	cshadow = NA_OptionalOutputArray(convolved, &cinfo, tFloat64, C_ARRAY, data);

	if (!kernel || !data || !cshadow)
		return PyErr_Format(_convolveError,
				    "Convolve2d: error normalizing array inputs.");

	if ((kinfo.ndim != 2) || (dinfo.ndim != 2))
		return PyErr_Format(_convolveError,
				    "Convolve2d: arrays must have 2 dimensions.");

	if (NA_ShapeMisMatch(&dinfo, &cinfo))
		return PyErr_Format(_convolveError,
				    "Convolve2d: data and output arrays need identitcal shapes.");

	Convolve2d(kinfo.shape[0], kinfo.shape[1], NA_OFFSETDATA(kinfo),
		   dinfo.shape[0], dinfo.shape[1], NA_OFFSETDATA(dinfo),
		   NA_OFFSETDATA(cinfo));

	Py_DECREF(kernel);
	Py_DECREF(data);

	/* Align, Byteswap, Contiguous, Typeconvert */
	return NA_ReturnOutput(convolved, cshadow);
}

static PyMethodDef _convolveMethods[] = {
    {"Convolve1d", Py_Convolve1d, METH_VARARGS},
    {"Convolve2d", Py_Convolve2d, METH_VARARGS},
    {NULL, NULL} /* Sentinel */
};

/* platform independent*/
#ifdef MS_WIN32
__declspec(dllexport)
#endif

void inithigh_level(void)
{
	PyObject *m, *d;
	m = Py_InitModule("high_level", _convolveMethods);
	d = PyModule_GetDict(m);
	_convolveError = PyErr_NewException("_high_level.error", NULL, NULL);
	PyDict_SetItemString(d, "error", _convolveError);
	import_libnumarray();
}

/*
 * Local Variables:
 * mode: C
 * c-file-style: "python"
 * End:
 */

==> py22-numarray-0.5.tbz: share/examples/py-numarray/convolve/high_levelmodule.c <==
#include "Python.h"

#include <stdio.h>
#include <math.h>
#include <signal.h>
#include <ctype.h>

#include "libnumarray.h"

static PyObject *_convolveError;

static void 
Convolve1d(long ksizex, Float64 *kernel, 
	   long dsizex, Float64 *data, 
	   Float64 *convolved)
{
	long xc;
	long halfk = ksizex/2;

	for(xc=0; xc<halfk; xc++)
		convolved[xc] = data[xc];

	for(xc=halfk; xc<dsizex-halfk; xc++) {
		long xk;
		double temp = 0;
		for (xk=0; xk<ksizex; xk++)
			temp += kernel[xk]*data[xc-halfk+xk];
		convolved[xc] = temp;
	}
		     
	for(xc=dsizex-halfk; xc<dsizex; xc++)
		convolved[xc] = data[xc];
}

static void 
Convolve2d(long krows, long kcols, Float64 *kernel, 
	   long drows, long dcols, Float64 *data, 
	   Float64 *convolved)
{
	long ki, kj, di, dj;
	long halfkrows = krows/2;
	long halfkcols = kcols/2;

	/* Copy the data in the half kernel "frame" straight through. */
	for(di=0; di<halfkrows; di++) {
		for(dj=0; dj<dcols; dj++)
			convolved[di*dcols+dj] = data[di*dcols+dj];
	}
	for(di=drows-halfkrows; di<drows; di++) {
		for(dj=0; dj<dcols; dj++)
			convolved[di*dcols+dj] = data[di*dcols+dj];
	}
	for(di=halfkrows; di<drows-halfkrows; di++) {
		for(dj=0; dj<halfkcols; dj++)
			convolved[di*dcols+dj] = data[di*dcols+dj];
	}
	for(di=halfkrows; di<drows-halfkrows; di++) {
		for(dj=dcols-halfkcols; dj<dcols; dj++)
			convolved[di*dcols+dj] = data[di*dcols+dj];
	}

	/* Do the convolution on the data within the frame. */
	for(di=halfkrows; di<drows-halfkrows; di++) {
		for(dj=halfkcols; dj<dcols-halfkcols; dj++) {
			Float64 temp = 0;
			for(ki=0; ki<krows; ki++) {
				long pi = di + ki - halfkrows;
				for(kj=0; kj<kcols; kj++) {
					long pj = dj + kj - halfkcols;
					temp += data[pi*dcols+pj] * 
						kernel[ki*kcols+kj];
				}
			}
			convolved[di*dcols+dj] = temp;
		}
	}
}


static PyObject *
Py_Convolve1d(PyObject *obj, PyObject *args)
{
	PyObject   *okernel, *odata, *oconvolved=Py_None;
	PyArrayObject *kernel, *data, *convolved;

	if (!PyArg_ParseTuple(args, "OO|O", &okernel, &odata, &oconvolved))
		return PyErr_Format(_convolveError, 
				    "Convolve1d: Invalid parameters.");

	/* Align, Byteswap, Contiguous, Typeconvert */
	kernel	= NA_InputArray(okernel, tFloat64, C_ARRAY);
	data	= NA_InputArray(odata, tFloat64, C_ARRAY);
	convolved = NA_OptionalOutputArray(oconvolved, tFloat64, C_ARRAY, data);

	if (!kernel || !data || !convolved)
		return PyErr_Format( _convolveError, 
			     "Convolve1d: error converting array inputs.");

	if ((kernel->nd != 1) || (data->nd != 1))
		return PyErr_Format(_convolveError,
			    "Convolve1d: arrays must have 1 dimensions.");

	if (!NA_ShapeEqual(data, convolved))
		return PyErr_Format(_convolveError,
			    "Convolve1d: data and output arrays need identitcal shapes.");

	Convolve1d(kernel->dimensions[0], 
		   NA_OFFSETDATA(kernel),
		   data->dimensions[0],   
		   NA_OFFSETDATA(data),
		   NA_OFFSETDATA(convolved));

	PyArray_XDECREF(kernel);
	PyArray_XDECREF(data);

	/* Align, Byteswap, Contiguous, Typeconvert */
	return NA_ReturnOutput(oconvolved, convolved);
}

static PyObject *
Py_Convolve2d(PyObject *obj, PyObject *args)
{
	PyObject   *okernel, *odata, *oconvolved=Py_None;
	PyArrayObject *kernel, *data, *convolved;

	if (!PyArg_ParseTuple(args, "OO|O", &okernel, &odata, &oconvolved))
		return PyErr_Format(_convolveError, 
				    "Convolve2d: Invalid parameters.");

	/* Align, Byteswap, Contiguous, Typeconvert */
	kernel	= NA_InputArray(okernel, tFloat64, C_ARRAY);
	data	= NA_InputArray(odata, tFloat64, C_ARRAY);
	convolved = NA_OptionalOutputArray(oconvolved, tFloat64, C_ARRAY, data);

	if (!kernel || !data || !convolved)
		return PyErr_Format( _convolveError, 
				     "Convolve2d: error converting array inputs.");

	if ((kernel->nd != 2) || (data->nd != 2))
		return PyErr_Format(_convolveError,
				    "Convolve2d: arrays must have 2 dimensions.");

	if (!NA_ShapeEqual(data, convolved))
		return PyErr_Format(_convolveError,
				    "Convolve2d: data and output arrays need identitcal shapes.");

	Convolve2d(kernel->dimensions[0], 
		   kernel->dimensions[1], 
		   NA_OFFSETDATA(kernel),
		   data->dimensions[0],   
		   data->dimensions[1],   
		   NA_OFFSETDATA(data),
		   NA_OFFSETDATA(convolved));

	PyArray_XDECREF(kernel);
	PyArray_XDECREF(data);

	/* Align, Byteswap, Contiguous, Typeconvert */
	return NA_ReturnOutput(oconvolved, convolved);
}

static PyMethodDef _convolveMethods[] = {
    {"Convolve1d", Py_Convolve1d, METH_VARARGS},
    {"Convolve2d", Py_Convolve2d, METH_VARARGS},
    {NULL, NULL} /* Sentinel */
};

/* platform independent*/
#ifdef MS_WIN32
__declspec(dllexport)
#endif

void inithigh_level(void)
{
	PyObject *m, *d;
	m = Py_InitModule("high_level", _convolveMethods);
	d = PyModule_GetDict(m);
	_convolveError = PyErr_NewException("_high_level.error", NULL, NULL);
	PyDict_SetItemString(d, "error", _convolveError);
	import_libnumarray();
}

/*
 * Local Variables:
 * mode: C
 * c-file-style: "python"
 * End:
 */
